@@ -66,7 +66,7 @@ See L<Net::ISC::DHCPd::Config::Role/regex>.
 
 =cut
 
-sub regex { qr{^\s* option \s+ (\S+) \s+ (.*) ;}x }
+our $regex = qr{^\s* option \s+ (\S+) \s+ (.*) ;}x;
 
 =head1 METHODS
 
