@@ -23,8 +23,6 @@ use Moose::Role;
 
 requires 'generate';
 
-my $COMMENT_RE = qr{^\s*\#\s*};
-
 =head1 ATTRIBUTES
 
 =head2 parent
@@ -49,21 +47,10 @@ The root node in the config tree.
 has root => (
     is => 'ro',
     isa => 'Object',
-    lazy => 1,
     weak_ref => 1,
-    builder => '_build_root',
+    default => sub { $_[0] },
 );
 
-sub _build_root {
-    my $obj = shift;
-
-    while(my $tmp = $obj->parent) {
-        blessed($obj = $tmp) eq 'Net::ISC::DHCPd::Config' and last;
-    }
-
-    return $obj;
-}
-
 =head2 depth
 
 Integer value that counts how far this node is from the root node.
@@ -181,21 +168,6 @@ THIS IS A STATIC METHOD.  SELF is not used.
 
 =cut
 
-=head2 endpoint
-
-Regex to search for before ending the current node block.
-Will not be used if the node does not have any possible L</children>.
-
-=cut
-
-has endpoint => (
-    is => 'ro',
-    isa => 'Maybe[RegexpRef]',
-    builder => '_build_endpoint',
-);
-
-sub _build_endpoint { qr" ^ \s* } \s* $ "x }
-
 has _filehandle => (
     is => 'ro',
     lazy_build => 1,
@@ -254,6 +226,9 @@ Used to convert input arguments to child nodes.
 
 sub BUILD {
     my($self, $args) = @_;
+    # skip the expensive method check if called through the parser.  The only
+    # time this code applies is with code like add_method( submethod => { } );
+    return if ($args->{'parse'});
     my $meta = $self->meta;
 
     for my $key (sort keys %$args) {
@@ -281,17 +256,20 @@ when possible.
 
 sub parse {
     my $self = shift;
-    my $linebuf = $_[1];
-    my $fh = $self->_filehandle;
-    my $endpoint = $self->endpoint;
-    my($n, $pos, @comments);
-    my $lines = '';
+    my $fh = $_[1];
+    my $linebuf = $_[2];
+    my($n, @comments);
+    my $lines;
     my $line_from_array=0;
+    # if $fh is a File::Temp or IO::File object then comparing on assignment
+    # above makes it convert to a number (refaddr), which takes 4µs/call
+    if (!defined($fh)) {
+        $fh = $self->_filehandle;
+    }
 
     LINE:
     while(1) {
         my $line;
-        $pos = $fh->getpos or die $!;
         if (defined($linebuf->[0])) {
             $line = pop(@{$linebuf});
             $line_from_array=1;
@@ -303,55 +281,30 @@ sub parse {
             # From here we need to preprocess the line to see if it can be broken
             # into multiple lines.  Something like group { option test; }
             # lines with comments can't be handled by this so we do them first
-            if($line =~ s/$COMMENT_RE//) {
-                push @comments, $line;
+            if($line =~ /^\s*\#\s*(.*)/) {
+                push @comments, $1;
                 next LINE;
             }
 
-            $line =~ s/ ([;\{\}])                                 # after semicolon or braces
-                        ([^;\n\r])                                # if there isn't a semicolon or return
-                        /$1\n$2/gx;                               # insert a newline
-
-            if ($line =~ /\n/) {
+            # after semicolon or braces if there isn't a semicolon or return insert a newline
+            if ($line =~ s/([;\{\}])([^;\n\r])/$1\n$2/g) {
                 push(@{$linebuf}, reverse split(/\n/, $line));
                 next LINE;
             }
         }
 
 
-        if($self->can('slurp')) {
-            my $action = $self->slurp($line); # next or last
-            if($action eq 'next') {
-                next LINE;
-            }
-            elsif($action eq 'last') {
-                last LINE;
-            }
-            elsif($action eq 'backtrack') {
-                if ($line_from_array) {
-                    push(@{$linebuf}, $line);
-                } else {
-                    $fh->setpos($pos);
-                    $n--;
-                }
-                last LINE;
-            }
-        }
-        elsif($line =~ /^\s*$/o) {
+        if ($line =~ /^(?:\s*|\s*\{\s*)$/) {
             next LINE;
         }
-        elsif($line =~ $endpoint) {
-            $self->captured_endpoint($1, $2, $3, $4); # urk...
+        elsif($line =~ /^\s*\}\s*$/) {
             next LINE if($self->root == $self);
             last LINE;
         }
-        elsif ($line =~ /^\s*{\s*$/) {
-            next LINE;
-        }
 
         # this is how we handle incomplete lines
         # we need a space for lines like 'option\ndomain-name-servers'
-        if ($lines =~ /\S$/) {
+        if ($lines) {
            $lines .= ' '.$line;
         } else {
             $lines = $line;
@@ -359,20 +312,23 @@ sub parse {
 
         CHILD:
         for my $child ($self->children) {
-            my $regex = $child->can('regex');
-            my @c = $lines =~ $regex->() or next CHILD;
+            no strict 'refs';
+            my $regex = ${"$child".'::regex'};
+            my @c = $lines =~ $regex or next CHILD;
             my $add = 'add_' .lc +($child =~ /::(\w+)$/)[0];
             my $method = $child->can('captured_to_args');
             my $args = $method->(@c);
             my $obj;
 
             $args->{'comments'} = [@comments];
+            $args->{'parse'} = 1;
             @comments = ();
-            $lines = '';
+            undef $lines;
             $obj = $self->$add($args);
+            $n += $obj->_parse_slurp($fh, $linebuf) if ($obj->can('slurp'));
 
             # the recursive statement is used for Include.pm
-            $n += $obj->parse('recursive', $linebuf) if(@_ = $obj->children);
+            $n += $obj->parse('recursive', $fh, $linebuf) if(@_ = $obj->children);
 
             next LINE;
         }
@@ -387,10 +343,9 @@ sub parse {
             next LINE;
         }
 
-
         if(warnings::enabled('net_isc_dhcpd_config_parse')) {
             warn sprintf qq[Could not parse "%s" at %s line %s\n],
-                $lines,
+                $line,
                 $self->root->file,
                 $fh->input_line_number
                 ;
@@ -400,6 +355,49 @@ sub parse {
     return $n ? $n : '0e0';
 }
 
+=head2 _parse_slurp
+
+This is a simplified parser for the slurp method.  It's only used when slurp
+is available in a child method.
+
+=cut
+
+
+sub _parse_slurp {
+    my $self = shift;
+    my $fh = shift;
+    my $linebuf = shift;
+    my($n, @comments);
+
+    LINE:
+    while(1) {
+        my $line;
+        if (defined($linebuf->[0])) {
+            $line = pop(@{$linebuf});
+        } else {
+            defined($line = readline $fh) or last LINE;
+            $n++;
+            chomp $line;
+        }
+
+
+        if($self->can('slurp')) {
+            my $action = $self->slurp($line); # next or last
+            if($action eq 'next') {
+                next LINE;
+            }
+            elsif($action eq 'last') {
+                last LINE;
+            }
+            elsif($action eq 'backtrack') {
+                push(@{$linebuf}, $line);
+                last LINE;
+            }
+        }
+    }
+    return $n;
+}
+
 =head2 captured_to_args
 
  $hash_ref = $self->captured_to_args(@list);
@@ -416,18 +414,6 @@ sub captured_to_args {
     return {};
 }
 
-=head2 captured_endpoint
-
-    $self->captured_endpoint(@list)
-
-Called when a L</endpoint> matches, with a list of captured strings.
-
-=cut
-
-sub captured_endpoint {
-    return;
-}
-
 =head2 create_children
 
 This method takes a list of classes, and creates builder method for
@@ -470,7 +456,7 @@ sub _set_children {
     my($self, $attr, $class, $children) = @_;
 
     for my $child (@$children) {
-        $child = $class->new(parent => $self, %$child) if(ref $child eq 'HASH');
+        $child = $class->new(parent => $self, root => $self->root, %$child) if(ref $child eq 'HASH');
     }
 
     @{ $self->_children } = @$children;
@@ -478,7 +464,7 @@ sub _set_children {
 
 sub _get_children_by_class {
     my($self, $class) = @_;
-    my @children = grep { $class eq blessed $_ } @{ $self->_children };
+    my @children = grep { $class eq ref $_ } @{ $self->_children };
 
     return wantarray ? @children : \@children;
 }
@@ -490,7 +476,7 @@ sub _add_child {
     my $children = $self->_children;
 
     if(ref $child eq 'HASH') {
-        $child = $class->new(parent => $self, %$child);
+        $child = $class->new(parent => $self, root => $self->root, %$child);
     }
 
     push @$children, $child;
@@ -507,7 +493,7 @@ sub _find_children {
 
     CHILD:
     for my $child (@{ $self->_children }) {
-        if($class ne blessed $child) {
+        if($class ne ref $child) {
             next CHILD;
         }
         for my $key (keys %$query) {
@@ -529,7 +515,7 @@ sub _remove_children {
 
     CHILD:
     while($i < @$children) {
-        if($class ne blessed $children->[$i]) {
+        if($class ne ref $children->[$i]) {
             next CHILD;
         }
         for my $key (keys %$query) {
