@@ -44,6 +44,7 @@ sub children {
         Net::ISC::DHCPd::Config::Range
         Net::ISC::DHCPd::Config::Filename
         Net::ISC::DHCPd::Config::Option
+        Net::ISC::DHCPd::Config::Class
         Net::ISC::DHCPd::Config::KeyValue
         Net::ISC::DHCPd::Config::Block
     /;
@@ -90,7 +91,7 @@ the ip address of this subnet.
 
 has address => (
     is => 'ro',
-    isa => 'NetAddr::IP',
+    isa => 'Object',
 );
 
 =head2 regex
@@ -99,7 +100,7 @@ See L<Net::ISC::DHCPd::Config/regex>.
 
 =cut
 
-sub regex { qr{^ \s* subnet \s+ (\S+) \s+ netmask \s+ (\S+) }x }
+our $regex = qr{^ \s* subnet \s+ (\S+) \s+ netmask \s+ (\S+) }x;
 
 =head1 METHODS
 
@@ -110,7 +111,7 @@ See L<Net::ISC::DHCPd::Config::Role/captured_to_args>.
 =cut
 
 sub captured_to_args {
-    return { address => NetAddr::IP->new(join "/", @_) };
+    return { address => NetAddr::IP->new(@_) };
 }
 
 =head2 generate
